@@ -1,5 +1,5 @@
 package MARC::Moose::Formater::UnimarcToMarc21;
-$MARC::Moose::Formater::UnimarcToMarc21::VERSION = '1.0.19';
+$MARC::Moose::Formater::UnimarcToMarc21::VERSION = '1.0.20';
 # ABSTRACT: Convert biblio record from UNIMARC to MARC21
 use Moose;
 
@@ -8,6 +8,7 @@ use utf8;
 
 extends 'MARC::Moose::Formater';
 
+use List::Util qw/ first /;
 use MARC::Moose::Field::Control;
 use MARC::Moose::Field::Std;
 
@@ -170,6 +171,12 @@ push @unchanged, [317, 561],
                  [337, 538],
                  [686, '084'];
 
+# Tags with non-filing indicator (pos 1 or 2)
+my $nonfiling_tags = [
+    [ qw/130 630 730 740 830/ ],
+    [ qw/240 242 243 245 440 830/ ],
+];
+
 # NSB/NSE characters
 my $ns_characters = [
     [ "\x08", "\x09" ],
@@ -177,6 +184,72 @@ my $ns_characters = [
 ];
 
 
+
+# Procedure 4 Title
+sub procedure_title {
+    my ($self, $subf) = @_;
+
+    my @sf;
+    my ($h_index) = (-1);
+    my @equivals = (
+        [ 'a', 'a' ],
+        [ 'j', 'f' ],
+        [ 'n', 'g' ],
+        [ 'h', 'n', '.' ],
+        [ 'k', 'f', '.' ],
+        [ 'l', 'k', '.' ],
+        [ 'm', 'l', '.' ],
+        [ 'q', 's', '.' ],
+        [ 'r', 'r', ',' ],
+        [ 's', 's', ',' ],
+        [ 't', 'o', ';' ],
+        [ 'u', 'r', ',' ],
+        [ 'x', 'x', ',' ],
+    );
+    for ( @$subf ) {
+        my ($letter, $value) = @$_;
+        if ( my $equival = first { $_->[0] eq $letter } @equivals ) {
+            my ($from, $to, $sep) = @$equival;
+            if ( $sep && @sf ) {
+                my $match = $sep;
+                $match = '\.' if $match eq '.';
+                if ( $sf[-1]->[0] !~ /$match$/ ) {
+                    $sf[-1]->[1] .= $sep;
+                }
+            }
+            push @sf, [ $to => $value ];
+        }
+        else {
+            given ($letter) {
+                when ( 'e' ) {
+                    next unless @sf; #FIXME warning required
+                    if ( $sf[-1][0] =~ /a|n|p/ ) {
+                        $sf[-1]->[1] .= ' :';
+                        push @sf, [ b => $value ];
+                    }
+                    else {
+                        $sf[-1]->[1] .= " : $value";
+                    }
+                }
+                when ( 'i' ) {
+                    if ( @sf ) {
+                        if ( $sf[-1]->[0] eq 'h' ) {
+                            $sf[-1]->[1] .= ','  if $sf[-1]->[1] !~ /,$/;
+                        }
+                        else {
+                            $sf[-1]->[1] .= '.';
+                        }
+                    }
+                    push @sf, [ p => $value ];
+                }
+            }
+        }
+    }
+
+    return \@sf;
+}
+
+
 override 'format' => sub {
     my ($self, $unimarc) = @_;
 
@@ -466,7 +539,8 @@ override 'format' => sub {
                         push @sf, [ n => $value ];
                     }
                     else {
-                        $sf[-1]->[1] .= ". $value";
+                        #$sf[-1]->[1] .= ". $value";
+                        push @sf, [ n => $value ];
                     }
                 }
                 when ( 'i' ) {
@@ -497,21 +571,6 @@ override 'format' => sub {
             }
             default { $ind1 = 1; }
         }
-        # Calculate second indicator based on UNIMARC NSB/NSE characters
-        unless ( $a_index == -1 ) {
-            my $a = $sf[$a_index][1];
-            for my $ns (@$ns_characters) {
-                my ($nsb, $nse) = @$ns;
-                next if $a !~ /^$nsb(.*)$nse/;
-                my $len = length($1);
-                $ind2 = $len  if $len < 10;
-                $a =~ s/$nsb//g;
-                $a =~ s/$nse//g;
-                $sf[$a_index][1] = $a;
-                last;
-            }
-        }
-
         $record->append( MARC::Moose::Field::Std->new(
             tag => '245', ind1 => $ind1, ind2 => $ind2,
             subf => \@sf ) );
@@ -820,6 +879,17 @@ override 'format' => sub {
             subf => [ [ a => join('  ', @a) ] ] ) );
     }
 
+    # 329 => 505
+    # This is French (CCfr) specific field without equivalent in MARC21
+    # Concatained into 505 field
+    for my $field ( $unimarc->field('359') ) {
+        my @a = map { $_->[1] } @{$field->subf};
+        $record->append( MARC::Moose::Field::Std->new(
+            tag => '505', ind1 => '0',
+            subf => [ [ a => join(' -- ', @a) ] ] ) );
+    }
+
+
     # 336 => 500
     for my $field ( $unimarc->field('336') ) {
         $record->append( MARC::Moose::Field::Std->new(
@@ -909,6 +979,25 @@ override 'format' => sub {
         }
     }
 
+    # 500 => 240 or 130
+    for my $field ( $unimarc->field('500') ) {
+        my ($ind1, $ind2) = ($field->ind1, $field->ind2);
+        my $tag = '240';
+        if ( $ind2 eq '0' ) {
+            $ind2 = 0;
+        }
+        elsif ( $ind2 eq '1' ) {
+            $tag = '130';
+            ($ind1, $ind2) = (0, ' ');
+        }
+        else {
+            ($ind1, $ind2) = (1, 0);
+        }
+        $record->append( MARC::Moose::Field::Std->new(
+            tag => $tag, ind1 => $ind1, ind2 => $ind2,
+            subf => $self->procedure_title($field->subf) ) );
+    }
+
     # 545 => 773, on passe t en a
     for my $field ( $unimarc->field('545') ) {
         $field->tag('773');
@@ -1116,11 +1205,52 @@ override 'format' => sub {
         }
     }
 
+    # Populate non-filing indicator based on UNIMARC NSB/NSE
+    {
+        my $first = 1;
+        for my $tags (@$nonfiling_tags) {
+            for my $tag (@$tags) {
+                for my $field ($record->field($tag)) {
+                    for (@{$field->subf}) {
+                        next if $_->[0] ne 'a';
+                        # Found Main title
+                        my $title = $_->[1];
+                        next unless $title;
+                        for my $ns (@$ns_characters) {
+                            my ($nsb, $nse) = @$ns;
+                            next if $title !~ /^$nsb(.*)$nse(.)/;
+                            my $len = length($1);
+                            $len++ if $2 eq ' ';
+                            $len = 0 if $len >= 10;
+                            $title =~ s/$nsb//g;
+                            $title =~ s/$nse//g;
+                            $_->[1] = $title;
+                            if ($first) { $field->ind1($len); }
+                            else        { $field->ind2($len); }
+                            last;
+                        }
+                        last;
+                    }
+                }
+            }
+            $first = 0;
+        }
+    }
+
     # Some fields are kept, as they are: 856, 801, 9xx
     if ( my @fields = $unimarc->field('801|856|9..') ) {
         $record->append(@fields)
     }
 
+    # Clean non-filing characters in all fields
+    for my $field (@{$record->fields}) {
+        next if $field->tag lt '010';
+        for (@{$field->subf} ) {
+            next if $_->[0] !~ /[a-z0-9]/;
+            $_->[1] =~ s/\x08|\x09//g;
+        }
+    }
+
     return $record;
 };
 
@@ -1140,7 +1270,7 @@ MARC::Moose::Formater::UnimarcToMarc21 - Convert biblio record from UNIMARC to M
 
 =head1 VERSION
 
-version 1.0.19
+version 1.0.20
 
 =head1 SYNOPSYS
 
@@ -1192,7 +1322,7 @@ Frédéric Demians <f.demians@tamil.fr>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Frédéric Demians.
+This software is copyright (c) 2015 by Frédéric Demians.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
